import models, events
import threading, time

board = None
cur_piece = nxt_piece = None
playing = paused = False
delay = 0.5
score = lines_cleared = 0
game_thread = None

game_listeners = []
score_listeners = []



def init_game(row, col):
    global board
    board = models.Board(width=col, height=row)

def start_game():
    global playing, cur_piece, nxt_piece
    global paused, score, delay, game_thread
    if not playing:
        cur_piece = None
        nxt_piece = models.Piece.get_random_piece(board)
        playing = True
        paused = False
        score = lines_cleared = 0
        delay = 0.5

        _fire_score_event()
        _fire_game_event(events.GameEvent(events.GameEvent.START))

        game_thread = GameThread()
        game_thread.start()

def stop_game():
    global playing
    playing = False
    _fire_game_event(events.GameEvent(events.GameEvent.END))

def pause_game():
    global playing, paused
    if playing:
        paused = True
def unpause_game():
    global playing, paused
    if playing:
        paused = False

def move(direction):
    global playing, paused, cur_piece
    result = False

    if (playing and (not paused) and cur_piece != None):
        result = cur_piece.move(direction)
        if (direction == 'DOWN' or direction == 'FALL'):
            if not result:
                cur_piece = None
    return result

def add_board_listener(listener):
    board.add_board_listener(listener)
def add_game_listener(listener):
    game_listeners.append(listener)
def add_score_listener(listener):
    score_listeners.append(listener)

def remove_board_listener(listener):
    board.remove_board_listener(listener)
def remove_game_listener(listener):
    game_listeners.remove(listener)
def remove_score_listener(listener):
    score_listeners.remove(listener)

def _fire_score_event():
    global score_listeners
    event = events.ScoreEvent(score, lines_cleared)
    for listener in score_listeners:
        listener.score_changed(event)

def _fire_game_event(event):
    global game_listeners
    for listener in game_listeners:
        listener.game_state_changed(event)


class GameThread(threading.Thread):

    def run(self):
        global playing, paused
        global cur_piece, nxt_piece
        global board, delay
        global score, lines_cleared
        while playing:
            if not paused:
                if cur_piece == None:
                    # clearing up filled rows
                    filled_rows = 0
                    r = board.height - 1
                    while r >= 0:
                        filled = True
                        # check if this row is all filled
                        for c in xrange(0, board.width):
                            if board.get_piece_type_at(c, r) == None:
                                filled = False
                                break
                        # remove filled row
                        if filled:
                            board.remove_row(r)
                            r += 1
                            filled_rows += 1
                            lines_cleared += 1
                        r -= 1
                    # at this point, all filled rows have been removed
                    if filled_rows > 0:
                        score += filled_rows * filled_rows * 10
                        _fire_score_event()
                    # randomize next piece
                    cur_piece = nxt_piece
                    nxt_piece = models.Piece.get_random_piece(board)
                    # place it on the board
                    cur_piece.center = (board.width/2, 1)
                    if not board.add_piece(cur_piece, True):
                        print 'error in trying to add piece:', cur_piece
                        stop_game()
                        break
                else:
                    # drop the current piece DOWN
                    move('DOWN')
                    
            if cur_piece != None:
                time.sleep(delay)



# UNIT TESTING
def __unit_test():
    init_game(row=20,col=10)
    class BL(object):
        def board_changed(self, event):
            global cur_piece
            print 'board changed! cur_piece:', cur_piece
    add_board_listener(BL())
    start_game()
    # pause game after 5 seconds
    time.sleep(5)
    pause_game()
    # unpause after 3 seconds
    time.sleep(3)
    unpause_game()
    # kill game after another 5 seconds
    time.sleep(5)
    stop_game()

if __name__ == '__main__':
    __unit_test()
