#! /usr/bin/env python2.5

"""

A simple implementation of tetris using wx and core tetris
functionality in tetris.core package.

"""

from core import game
import conf
import wx


class BoardGUI(wx.Panel):
    """This panel will hold a GUI representation of the tetris board."""
    def __init__(self, parent, *args, **kwargs):
        wx.Panel.__init__(self, parent, *args, **kwargs)
        self._boardref = None

        # hook the paint event
        self.UpdateBuffer()
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def UpdateBuffer(self):
        size = self.GetClientSizeTuple()
        self._Buffer = wx.EmptyBitmap(*size)
        dc = wx.BufferedDC(wx.ClientDC(self), self._Buffer)
        self.Draw(dc)

    def OnPaint(self, evt):
        # draw the buffer to the screen
        dc = wx.BufferedPaintDC(self, self._Buffer)

    def Draw(self, dc):
        # set up the device context for painting
        dc.BeginDrawing()
        dc.SetBackground(wx.Brush("Blue"))
        dc.Clear()
        dc.SetPen(wx.Pen("BLACK", 1))

        width = self.Size.width
        height = self.Size.height

        # draw the board pieces if boardref is not null
        if self._boardref != None:
            cols = self._boardref.width
            rows = self._boardref.height

            for c in range(0, cols):
                for r in range(0, rows):
                    piecet = self._boardref.get_piece_type_at(c, r)
                    if piecet != None:
                        # set the block brush color
                        dc.SetBrush(wx.Brush(conf.PieceColor[piecet], wx.SOLID))
                        # set the settings for the block
                        w = width / cols - 1
                        h = height / rows - 1
                        x = c * width / cols + 1
                        y = r * height / rows + 1
                        # now, actually draw the block
                        dc.DrawRectangle(x, y, w, h)
        dc.EndDrawing()

    def board_changed(self, evt):
        """This class is also a Board listener."""
        self._boardref = evt.source
        self.UpdateBuffer()
        self.Refresh()
        #self.Update()


class Tetris(object):
    """The tetris game."""

    app = wx.App()

    def __init__(self):
        object.__init__(self)
        # initialize game module
        game.init_game(col=10, row=20)
        # initialize the components
        self.frame = wx.Frame(None, -1, 'wxtetris', size=(300, 700))
        self.boardgui = BoardGUI(self.frame, -1, size=(300, 600))
        self.score = wx.StaticText(self.frame, -1, label='0')
        self.start = wx.Button(self.frame, -1, label='start')
        # layout
        self.layout_components()
        # set event handlers
        self.set_event_handlers()
    
    def layout_components(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.start)
        sizer.Add(self.boardgui)
        sizer.Add(self.score)

        self.frame.SetAutoLayout(True)
        self.frame.SetSizer(sizer)
        self.frame.Layout()

    def set_event_handlers(self):
        self.start.Bind(wx.EVT_BUTTON, self.button_click)
        self.boardgui.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        game.add_board_listener(self.boardgui)
        game.add_score_listener(self)

    def start_tetris(self):
        self.frame.Show()
        self.app.MainLoop()

    def OnKeyDown(self, evt):
        keycode = evt.GetKeyCode()
        if keycode == wx.WXK_UP:
            dir = 'ROTATE'
        elif keycode == wx.WXK_DOWN:
            dir = 'DOWN'
        elif keycode == wx.WXK_RIGHT:
            dir = 'RIGHT'
        elif keycode == wx.WXK_LEFT:
            dir = 'LEFT'
        elif keycode == wx.WXK_SPACE:
            dir = 'FALL'
        else:
            dir = None

        if dir:
            game.move(dir)

    def button_click(self, evt):
        game.start_game()

    def score_changed(self, evt):
        # set score text
        self.score.SetLabel(label=str(game.score))
        # adjust delay
        delay = game.delay
        lines = evt.lines_cleared
        if lines >= 50:
            delay = 120
        elif lines >= 40:
            delay = 150
        elif lines >= 30:
            delay = 200
        elif lines >= 20:
            delay = 300
        elif lines >= 10:
            delay = 400
        game.delay = delay


if __name__ == '__main__':
    t = Tetris()
    t.start_tetris()
